/**
 * Copyright 2006 Envoi Solutions LLC
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.codehaus.jettison.util;

import java.util.ArrayList;
import java.util.EmptyStackException;

public class FastStack extends ArrayList {
    public void push(Object o) {
        add(o);
    }

    public Object pop() {
        if (empty())
            throw new EmptyStackException();

        return remove(size() - 1);
    }

    public boolean empty() {
        return size() == 0;
    }

    public Object peek() {
        if (empty())
            throw new EmptyStackException();

        return get(size() - 1);
    }
}
